@@ -1,11 +1,11 @@
-use crate::TableAttr;
+use crate::TableMeta;
 use proc_macro2::TokenStream;
 use quote::{quote, ToTokens};
 use syn::punctuated::Punctuated;
 use syn::token::Comma;
 use syn::{DeriveInput, Field};
 
-/// Given the fields of a PartialModel struct, return the quoted code.
+/// Given the fields of a ModelBuilder struct, return the quoted code.
 fn generate_query_binding_code_for_partial_model(
     fields: &Punctuated<Field, Comma>,
 ) -> impl Iterator<Item = TokenStream> + '_ {
@@ -32,6 +32,7 @@ fn get_field_names(fields: &Punctuated<Field, Comma>) -> impl Iterator<Item = St
     fields.iter().map(|f| f.ident.as_ref().unwrap().to_string())
 }
 
+/// bool whether the given type is `String`
 fn ty_is_string(ty: &syn::Type) -> bool {
     let p = match ty {
         syn::Type::Path(p) => p,
@@ -41,16 +42,19 @@ fn ty_is_string(ty: &syn::Type) -> bool {
 }
 
 pub trait OrmliteCodegen {
-    fn impl_TableMeta(ast: &DeriveInput, attr: &TableAttr) -> TokenStream {
+    fn database() -> TokenStream;
+    fn placeholder() -> TokenStream;
+    fn raw_placeholder() -> Box<dyn Iterator<Item = String>>;
+
+    fn impl_TableMeta(ast: &DeriveInput, attr: &TableMeta) -> TokenStream {
         let model = &ast.ident;
         let table_name = &attr.table_name;
-        let id = &attr.primary_key_column;
+        let id = &attr.primary_key;
         let fields = crate::util::get_fields(ast);
         let field_names = get_field_name_tokens(fields);
         let n_fields = fields.len();
 
         quote! {
-            impl #model {}
             impl ::ormlite::model::TableMeta for #model {
                 fn table_name() -> &'static str {
                     #table_name
@@ -71,7 +75,7 @@ pub trait OrmliteCodegen {
         }
     }
 
-    fn impl_Model__insert(ast: &DeriveInput, attr: &TableAttr) -> TokenStream {
+    fn impl_Model__insert(ast: &DeriveInput, attr: &TableMeta) -> TokenStream {
         let box_future = crate::util::box_future();
         let db = Self::database();
         let fields = crate::util::get_fields(ast);
@@ -96,9 +100,12 @@ pub trait OrmliteCodegen {
         });
 
         quote! {
-            fn insert(self, db: &mut <#db as ::sqlx::Database>::Connection) -> #box_future<::ormlite::Result<Self>> {
+            fn insert<'e, E>(self, db: E) -> #box_future<'e, ::ormlite::Result<Self>>
+            where
+                E: 'e + ::ormlite::export::Executor<'e, Database = #db>,
+            {
                 Box::pin(async move {
-                    let mut q = sqlx::query_as::<#db, Self>(#query);
+                    let mut q = ::ormlite::export::query_as::<#db, Self>(#query);
                     #(#query_bindings)*
                     q.fetch_one(db)
                         .await
@@ -108,13 +115,13 @@ pub trait OrmliteCodegen {
         }
     }
 
-    fn impl_Model__update_all_fields(ast: &DeriveInput, attr: &TableAttr) -> TokenStream {
+    fn impl_Model__update_all_fields(ast: &DeriveInput, attr: &TableMeta) -> TokenStream {
         let box_future = crate::util::box_future();
         let db = Self::database();
         let fields = crate::util::get_fields(ast);
         let mut placeholder = Self::raw_placeholder();
         let update_clause = get_field_names(fields)
-            .filter(|f| f != &attr.primary_key_column)
+            .filter(|f| f != &attr.primary_key)
             .map(|f| format!("{} = {}", f, placeholder.next().unwrap()))
             .collect::<Vec<_>>()
             .join(", ");
@@ -123,14 +130,14 @@ pub trait OrmliteCodegen {
             "UPDATE {} SET {} WHERE {} = {} RETURNING *",
             attr.table_name,
             update_clause,
-            attr.primary_key_column,
+            attr.primary_key,
             placeholder.next().unwrap()
         );
 
-        let id_field = quote::format_ident!("{}", attr.primary_key_column);
+        let id_field = quote::format_ident!("{}", attr.primary_key);
         let query_bindings = fields
             .iter()
-            .filter(|f| &f.ident.as_ref().unwrap().to_string() != &attr.primary_key_column)
+            .filter(|f| &f.ident.as_ref().unwrap().to_string() != &attr.primary_key)
             .map(|f| {
                 let name = &f.ident;
                 quote! {
@@ -139,9 +146,12 @@ pub trait OrmliteCodegen {
             });
 
         quote! {
-            fn update_all_fields(self, db: &mut <#db as ::sqlx::Database>::Connection) -> #box_future<::ormlite::Result<Self>> {
+            fn update_all_fields<'e, E>(self, db: E) -> #box_future<'e, ::ormlite::Result<Self>>
+            where
+                E: 'e + ::ormlite::export::Executor<'e, Database = #db>,
+            {
                 Box::pin(async move {
-                    let mut q = sqlx::query_as::<_, Self>(#query);
+                    let mut q = ::ormlite::export::query_as::<_, Self>(#query);
                     #(#query_bindings)*
                     q.bind(self.#id_field)
                         .fetch_one(db)
@@ -152,23 +162,26 @@ pub trait OrmliteCodegen {
         }
     }
 
-    fn impl_Model__delete(_ast: &DeriveInput, attr: &TableAttr) -> TokenStream {
+    fn impl_Model__delete(_ast: &DeriveInput, attr: &TableMeta) -> TokenStream {
         let mut placeholder = Self::raw_placeholder();
 
         let query = format!(
             "DELETE FROM {} WHERE {} = {}",
             attr.table_name,
-            attr.primary_key_column,
+            attr.primary_key,
             placeholder.next().unwrap()
         );
 
         let box_future = crate::util::box_future();
         let db = Self::database();
-        let id_field = quote::format_ident!("{}", attr.primary_key_column);
+        let id_field = quote::format_ident!("{}", attr.primary_key);
         quote! {
-           fn delete(self, db: &mut <#db as ::sqlx::Database>::Connection) -> #box_future<::ormlite::Result<()>> {
+            fn delete<'e, E>(self, db: E) -> #box_future<'e, ::ormlite::Result<()>>
+            where
+                E: 'e + ::ormlite::export::Executor<'e, Database = #db>
+            {
                 Box::pin(async move {
-                    let row = ::sqlx::query(#query)
+                    let row = ::ormlite::export::query(#query)
                         .bind(self.#id_field)
                         .execute(db)
                         .await
@@ -183,26 +196,24 @@ pub trait OrmliteCodegen {
         }
     }
 
-    fn impl_Model__get_one(_ast: &DeriveInput, attr: &TableAttr) -> TokenStream {
+    fn impl_Model__get_one(_ast: &DeriveInput, attr: &TableMeta) -> TokenStream {
         let mut placeholder = Self::raw_placeholder();
 
         let query = format!(
             "SELECT * FROM {} WHERE {} = {}",
             attr.table_name,
-            attr.primary_key_column,
+            attr.primary_key,
             placeholder.next().unwrap()
         );
 
         let db = Self::database();
         let box_future = crate::util::box_future();
         quote! {
-            fn get_one<'db, 'arg: 'db, T>(
-                id: T,
-                db: &'db mut <#db as ::sqlx::Database>::Connection,
-            ) ->
-                #box_future<'db, ::ormlite::Result<Self>>
-                where
-                    T: 'arg + Send + for<'r> ::sqlx::Encode<'r, #db> + ::sqlx::Type<#db>,
+            fn get_one<'e, 'a, Arg, E>(id: Arg, db: E) -> #box_future<'e, ::ormlite::Result<Self>>
+            where
+                'a: 'e,
+                Arg: 'a + Send + for<'r> ::sqlx::Encode<'r, #db> + ::sqlx::Type<#db>,
+                E: 'e + ::ormlite::export::Executor<'e, Database = #db>
             {
                 Box::pin(async move {
                     ::sqlx::query_as::<#db, Self>(#query)
@@ -215,7 +226,7 @@ pub trait OrmliteCodegen {
         }
     }
 
-    fn impl_Model(ast: &DeriveInput, attr: &TableAttr) -> TokenStream {
+    fn impl_Model(ast: &DeriveInput, attr: &TableMeta) -> TokenStream {
         let db = Self::database();
         let model = &ast.ident;
 
@@ -226,7 +237,6 @@ pub trait OrmliteCodegen {
         let impl_Model__get_one = Self::impl_Model__get_one(ast, attr);
 
         quote! {
-            // impl #model : ::ormlite::model::Model<#db> {}
             impl ::ormlite::model::Model<#db> for #model {
                 #impl_Model__insert
                 #impl_Model__update_all_fields
@@ -241,12 +251,12 @@ pub trait OrmliteCodegen {
         }
     }
 
-    fn impl_BuildsQueryBuilder(ast: &DeriveInput, attr: &TableAttr) -> TokenStream {
+    fn impl_HasQueryBuilder(ast: &DeriveInput, attr: &TableMeta) -> TokenStream {
         let table_name = &attr.table_name;
         let model = &ast.ident;
         let db = Self::database();
         quote! {
-            impl ::ormlite::model::BuildsQueryBuilder<#db, ::std::boxed::Box<dyn Iterator<Item=String>>> for #model {
+            impl ::ormlite::model::HasQueryBuilder<#db, ::std::boxed::Box<dyn Iterator<Item=String>>> for #model {
                 fn select<'args>() -> ::ormlite::SelectQueryBuilder<'args, #db, Self, Box<dyn Iterator<Item=String>>> {
                     ::ormlite::SelectQueryBuilder::default()
                         .column(&format!("{}.*", #table_name))
@@ -255,11 +265,11 @@ pub trait OrmliteCodegen {
         }
     }
 
-    fn impl_BuildsPartialModel(ast: &DeriveInput, _attr: &TableAttr) -> TokenStream {
+    fn impl_HasModelBuilder(ast: &DeriveInput, _attr: &TableMeta) -> TokenStream {
         let model = &ast.ident;
         let partial_model = quote::format_ident!("Partial{}", model.to_string());
         quote! {
-            impl<'a> ormlite::model::BuildsPartialModel<'a, #partial_model<'a>> for #model {
+            impl<'a> ormlite::model::HasModelBuilder<'a, #partial_model<'a>> for #model {
                 fn build() -> #partial_model<'a> {
                     #partial_model::default()
                 }
@@ -273,12 +283,9 @@ pub trait OrmliteCodegen {
         }
     }
 
-    fn struct_PartialModel(ast: &DeriveInput, _attr: &TableAttr) -> TokenStream {
-        let base_ident = &ast.ident;
-        let partial_ident = syn::Ident::new(
-            &("Partial".to_string() + ast.ident.to_string().as_str()),
-            proc_macro2::Span::call_site(),
-        );
+    fn struct_ModelBuilder(ast: &DeriveInput, _attr: &TableMeta) -> TokenStream {
+        let model = &ast.ident;
+        let model_builder = quote::format_ident!("Partial{}", model.to_string());
         let pub_marker = &ast.vis;
 
         let fields = crate::util::get_fields(ast);
@@ -327,12 +334,12 @@ pub trait OrmliteCodegen {
         });
 
         quote! {
-            #pub_marker struct #partial_ident<'a> {
+            #pub_marker struct #model_builder<'a> {
                 #(#settable,)*
-                updating: Option<&'a #base_ident>,
+                updating: Option<&'a #model>,
             }
 
-            impl<'a> std::default::Default for #partial_ident<'a> {
+            impl<'a> std::default::Default for #model_builder<'a> {
                 fn default() -> Self {
                     Self {
                         #(#fields_none,)*
@@ -341,7 +348,7 @@ pub trait OrmliteCodegen {
                 }
             }
 
-            impl<'a> #partial_ident<'a> {
+            impl<'a> #model_builder<'a> {
                 #(#methods)*
 
                 fn modified_fields(&self) -> Vec<&'static str> {
@@ -353,7 +360,7 @@ pub trait OrmliteCodegen {
         }
     }
 
-    fn impl_PartialModel__insert(ast: &DeriveInput, attr: &TableAttr) -> TokenStream {
+    fn impl_ModelBuilder__insert(ast: &DeriveInput, attr: &TableMeta) -> TokenStream {
         let box_future = crate::util::box_future();
         let db = Self::database();
         let placeholder = Self::placeholder();
@@ -366,7 +373,10 @@ pub trait OrmliteCodegen {
         let bind_parameters = generate_query_binding_code_for_partial_model(fields);
 
         quote! {
-            fn insert<'db: 'a>(self, db: &'db mut <#db as ::sqlx::Database>::Connection) -> #box_future<'a, ::ormlite::Result<Self::Model>> {
+            fn insert<'e: 'a, E>(self, db: E) -> #box_future<'a, ::ormlite::Result<Self::Model>>
+            where
+                E: 'e + ::ormlite::export::Executor<'e, Database = #db>,
+            {
                 Box::pin(async move {
                     let mut placeholder = #placeholder;
                     let set_fields = self.modified_fields();
@@ -375,7 +385,7 @@ pub trait OrmliteCodegen {
                         set_fields.join(", "),
                         set_fields.iter().map(|_| placeholder.next().unwrap()).collect::<Vec<_>>().join(", "),
                     );
-                    let mut q = ::sqlx::query_as::<#db, Self::Model>(&query);
+                    let mut q = ::ormlite::export::query_as::<#db, Self::Model>(&query);
                     #(#bind_parameters)*
                     q.fetch_one(db)
                         .await
@@ -385,21 +395,24 @@ pub trait OrmliteCodegen {
         }
     }
 
-    fn impl_PartialModel__update(ast: &DeriveInput, attr: &TableAttr) -> TokenStream {
+    fn impl_ModelBuilder__update(ast: &DeriveInput, attr: &TableMeta) -> TokenStream {
         let fields = crate::util::get_fields(ast);
 
         let query = format!(
             "UPDATE {} SET {{}} WHERE {} = {{}} RETURNING *",
-            attr.table_name, attr.primary_key_column,
+            attr.table_name, attr.primary_key,
         );
 
         let db = Self::database();
         let box_future = crate::util::box_future();
         let placeholder = Self::placeholder();
         let bind_update = generate_query_binding_code_for_partial_model(fields);
-        let id = quote::format_ident!("{}", attr.primary_key_column);
+        let id = quote::format_ident!("{}", attr.primary_key);
         quote! {
-            fn update<'db: 'a>(self, db: &'db mut <#db as ::sqlx::Database>::Connection) -> #box_future<'a, ::ormlite::Result<Self::Model>> {
+            fn update<'e: 'a, E>(self, db: E) -> #box_future<'a, ::ormlite::Result<Self::Model>>
+            where
+                E: 'e + ::ormlite::export::Executor<'e, Database = #db>,
+            {
                 Box::pin(async move {
                     let mut placeholder = #placeholder;
                     let set_fields = self.modified_fields();
@@ -407,14 +420,14 @@ pub trait OrmliteCodegen {
                         #query,
                         set_fields.into_iter().map(|f| format!("{} = {}", f, placeholder.next().unwrap())).collect::<Vec<_>>().join(", "),
                         self.updating
-                            .expect("Tried to call PartialModel::update(), but the PartialModel \
+                            .expect("Tried to call ModelBuilder::update(), but the ModelBuilder \
                             has no reference to what model to update. You might have called \
                             something like: `<Model>::build().update(&mut db)`. A partial update \
                             looks something like \
                             `<model instance>.update_partial().update(&mut db)`.")
                             .#id
                     );
-                    let mut q = ::sqlx::query_as::<#db, Self::Model>(&query);
+                    let mut q = ::ormlite::export::query_as::<#db, Self::Model>(&query);
                     #(#bind_update)*
                     q.fetch_one(db)
                         .await
@@ -423,24 +436,103 @@ pub trait OrmliteCodegen {
             }
         }
     }
-    fn impl_PartialModel(ast: &DeriveInput, attr: &TableAttr) -> TokenStream {
+    fn impl_ModelBuilder(ast: &DeriveInput, attr: &TableMeta) -> TokenStream {
         let model = &ast.ident;
         let db = Self::database();
         let partial_model = quote::format_ident!("Partial{}", model.to_string());
 
-        let impl_PartialModel__insert = Self::impl_PartialModel__insert(ast, attr);
-        let impl_PartialModel__update = Self::impl_PartialModel__update(ast, attr);
+        let impl_ModelBuilder__insert = Self::impl_ModelBuilder__insert(ast, attr);
+        let impl_ModelBuilder__update = Self::impl_ModelBuilder__update(ast, attr);
 
         quote! {
-            impl<'a> ::ormlite::model::PartialModel<'a, #db> for #partial_model<'a> {
+            impl<'a> ::ormlite::model::ModelBuilder<'a, #db> for #partial_model<'a> {
                 type Model = #model;
-                #impl_PartialModel__insert
-                #impl_PartialModel__update
+                #impl_ModelBuilder__insert
+                #impl_ModelBuilder__update
             }
         }
     }
 
-    fn database() -> TokenStream;
-    fn placeholder() -> TokenStream;
-    fn raw_placeholder() -> Box<dyn Iterator<Item = String>>;
+    fn struct_InsertModel(ast: &DeriveInput, attr: &TableMeta) -> TokenStream {
+        if attr.insert_struct.is_none() {
+            return quote! {};
+        }
+        let model_builder = quote::format_ident!("{}", attr.insert_struct.as_ref().unwrap());
+        let pub_marker = &ast.vis;
+        let fields = crate::util::get_fields(ast);
+        let struct_fields = fields
+            .iter()
+            .zip(attr.columns.iter())
+            .filter(|(_f, col_meta)| !col_meta.has_database_default)
+            .map(|(f, _)| {
+                let name = &f.ident;
+                let ty = &f.ty;
+                quote! { #name: #ty }
+            });
+
+        quote! {
+            #pub_marker struct #model_builder {
+                #(#struct_fields,)*
+            }
+        }
+    }
+
+    fn impl_InsertModel(ast: &DeriveInput, attr: &TableMeta) -> TokenStream {
+        if attr.insert_struct.is_none() {
+            return quote! {};
+        }
+        let box_future = crate::util::box_future();
+        let model = &ast.ident;
+        let db = Self::database();
+        let insert_model = quote::format_ident!("{}", attr.insert_struct.as_ref().unwrap());
+        let fields = attr
+            .columns
+            .iter()
+            .filter(|col_meta| !col_meta.has_database_default)
+            .map(|col_meta| col_meta.column_name.clone())
+            .collect::<Vec<_>>()
+            .join(",");
+        let mut placeholder = Self::raw_placeholder();
+        let placeholders = attr
+            .columns
+            .iter()
+            .filter(|col_meta| !col_meta.has_database_default)
+            .map(|_| placeholder.next().unwrap())
+            .collect::<Vec<_>>()
+            .join(",");
+        let query = format!(
+            "INSERT INTO {} ({}) VALUES ({}) RETURNING *",
+            attr.table_name, fields, placeholders,
+        );
+
+        let query_bindings = attr
+            .columns
+            .iter()
+            .filter(|col_meta| !col_meta.has_database_default)
+            .map(|f| {
+                let name = quote::format_ident!("{}", f.column_name);
+                quote! {
+                    q = q.bind(self.#name);
+                }
+            });
+
+        quote! {
+            impl<'a> Insertable<'a, #db> for #insert_model {
+                type Model = #model;
+
+                fn insert<'e: 'a, E>(self, db: E) -> #box_future<'a, ::ormlite::Result<Self::Model>>
+                where
+                    E: 'e + ::ormlite::export::Executor<'e, Database = #db>,
+                {
+                    Box::pin(async move {
+                        let mut q = ::ormlite::export::query_as::<#db, Self::Model>(#query);
+                        #(#query_bindings)*
+                        q.fetch_one(db)
+                            .await
+                            .map_err(::ormlite::Error::from)
+                    })
+                }
+            }
+        }
+    }
 }
